@@ -16,7 +16,7 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
  *
- * $Id: bacon-video-widget-xine.c,v 1.229 2006/04/28 10:45:07 hadess Exp $
+ * $Id: bacon-video-widget-xine.c,v 1.235 2006/07/06 18:51:22 hadess Exp $
  *
  * the xine engine in a widget - implementation
  */
@@ -157,10 +157,13 @@ struct BaconVideoWidgetPrivate {
 	float seek_dest;
 	gint64 seek_dest_time;
 
+	/* Logo */
+	gboolean logo_mode;
+	GdkPixbuf *logo_pixbuf;
+
 	/* Other stuff */
 	int xpos, ypos;
 	gboolean can_dvd, can_vcd, can_cdda;
-	gboolean logo_mode;
 	guint tick_id;
 	gboolean have_xvidmode;
 	gboolean auto_resize;
@@ -1105,6 +1108,8 @@ bacon_video_widget_realize (GtkWidget *widget)
 
 	bvw->priv->video_window = widget->window;
 
+	widget->style = gtk_style_attach (widget->style, widget->window);
+
 	/* Set a black background */
 	gdk_draw_rectangle (widget->window, widget->style->black_gc, TRUE,
 			attr.x, attr.y,
@@ -1753,18 +1758,65 @@ static gboolean
 bacon_video_widget_expose (GtkWidget *widget, GdkEventExpose *event)
 {
 	BaconVideoWidget *bvw = (BaconVideoWidget *) widget;
-	XExposeEvent *expose;
+	gboolean draw_logo, has_video;
 
-	if (event->count != 0)
-		return FALSE;
+	/* if there's only audio and no visualisation, draw the logo as well */
+	has_video = xine_get_stream_info(bvw->priv->stream,
+			XINE_STREAM_INFO_HAS_VIDEO);
+	draw_logo = !has_video && !bvw->priv->show_vfx;
 
-	expose = g_new0 (XExposeEvent, 1);
-	expose->count = event->count;
+	if (bvw->priv->logo_mode == FALSE && draw_logo == FALSE) {
+		XExposeEvent *expose;
 
-	xine_port_send_gui_data (bvw->priv->vo_driver,
-			XINE_GUI_SEND_EXPOSE_EVENT, expose);
+		if (event->count != 0)
+			return FALSE;
 
-	g_free (expose);
+		expose = g_new0 (XExposeEvent, 1);
+		expose->count = event->count;
+
+		xine_port_send_gui_data (bvw->priv->vo_driver,
+				XINE_GUI_SEND_EXPOSE_EVENT, expose);
+
+		g_free (expose);
+	} else {
+		int s_width, s_height, w_width, w_height;
+		GdkPixbuf *logo = NULL;
+		gfloat ratio;
+
+		/* Start with a nice black canvas */
+		gdk_draw_rectangle (widget->window, widget->style->black_gc,
+				TRUE, 0, 0,
+				widget->allocation.width,
+				widget->allocation.height);
+
+		if (bvw->priv->logo_pixbuf == NULL)
+			return FALSE;
+
+		s_width = gdk_pixbuf_get_width (bvw->priv->logo_pixbuf);
+		s_height = gdk_pixbuf_get_height (bvw->priv->logo_pixbuf);
+		w_width = widget->allocation.width;
+		w_height = widget->allocation.height;
+
+		if ((gfloat) w_width / s_width > (gfloat) w_height / s_height) {
+			ratio = (gfloat) w_height / s_height;
+		} else {
+			ratio = (gfloat) w_width / s_width;
+		}
+
+		s_width *= ratio;
+		s_height *= ratio;
+
+		logo = gdk_pixbuf_scale_simple (bvw->priv->logo_pixbuf,
+				s_width, s_height, GDK_INTERP_BILINEAR);
+
+		gdk_draw_pixbuf (widget->window, widget->style->fg_gc[0], logo,
+				0, 0,
+				(w_width - s_width) / 2,
+				(w_height - s_height) / 2,
+				s_width, s_height, GDK_RGB_DITHER_NONE, 0, 0);
+
+		gdk_pixbuf_unref (logo);
+	}
 
 	return FALSE;
 }
@@ -1958,8 +2010,12 @@ show_vfx_update (BaconVideoWidget *bvw, gboolean show_visuals)
 	if (enable == FALSE) {
 		audio_source = xine_get_audio_source (bvw->priv->stream);
 		if (xine_post_wire_audio_port (audio_source,
-					bvw->priv->ao_driver))
+					bvw->priv->ao_driver)) {
 			bvw->priv->using_vfx = FALSE;
+
+			/* Queue a redraw of the widget */
+			gtk_widget_queue_draw (GTK_WIDGET (bvw));
+		}
 		if (bvw->priv->vis != NULL) {
 			xine_post_dispose (bvw->priv->xine,
 					bvw->priv->vis);
@@ -1968,8 +2024,12 @@ show_vfx_update (BaconVideoWidget *bvw, gboolean show_visuals)
 	} else {
 		audio_source = xine_get_audio_source (bvw->priv->stream);
 		if (xine_post_wire_audio_port (audio_source,
-					bvw->priv->vis->audio_input[0]))
+					bvw->priv->vis->audio_input[0])) {
 			bvw->priv->using_vfx = TRUE;
+
+			/* Queue a redraw of the widget */
+			gtk_widget_queue_draw (GTK_WIDGET (bvw));
+		}
 	}
 }
 
@@ -2409,18 +2469,27 @@ bacon_video_widget_set_logo_mode (BaconVideoWidget *bvw, gboolean logo_mode)
 	g_return_if_fail (bvw->priv->xine != NULL);
 
 	bvw->priv->logo_mode = logo_mode;
+
+	/* Queue a redraw of the widget */
+	gtk_widget_queue_draw (GTK_WIDGET (bvw));
 }
 
 void
 bacon_video_widget_set_logo (BaconVideoWidget *bvw, char *filename)
 {
+	GError *err = NULL;
+
 	g_return_if_fail (bvw != NULL);
 	g_return_if_fail (BACON_IS_VIDEO_WIDGET(bvw));
 	g_return_if_fail (bvw->priv->xine != NULL);
 	g_return_if_fail (filename != NULL);
+	g_return_if_fail (bvw->priv->logo_pixbuf == NULL);
 
-	if (bacon_video_widget_open (bvw, filename, NULL) != FALSE) {
-		bacon_video_widget_play (bvw, NULL);
+	bvw->priv->logo_pixbuf = gdk_pixbuf_new_from_file (filename, &err);
+	if (err) {
+		g_warning ("Couldn't open logo image: %s",
+				err->message ? err->message : "No reason");
+		g_error_free (err);
 	}
 }
 
@@ -3058,6 +3127,26 @@ bacon_video_widget_set_subtitle_font (BaconVideoWidget *bvw, const char *font)
 	//FIXME
 }
 
+void
+bacon_video_widget_set_subtitle_encoding (BaconVideoWidget *bvw, const char *encoding)
+{
+	xine_cfg_entry_t entry;
+	char *lower;
+
+	g_return_if_fail (bvw != NULL);
+	g_return_if_fail (BACON_IS_VIDEO_WIDGET (bvw));
+	g_return_if_fail (bvw->priv->xine != NULL);
+	g_return_if_fail (encoding != NULL);
+
+	lower = g_ascii_strdown (encoding, -1);
+
+	bvw_config_helper_string (bvw->priv->xine,
+			"subtitles.separate.src_encoding", lower, &entry);
+	entry.str_value = (char *) lower;
+	xine_config_update_entry (bvw->priv->xine, &entry);
+	g_free (lower);
+}
+
 void
 bacon_video_widget_set_video_device (BaconVideoWidget *bvw, const char *path)
 {
