@@ -46,6 +46,7 @@
 #define MIME_READ_CHUNK_SIZE 1024
 #define RECURSE_LEVEL_MAX 4
 #define DIR_MIME_TYPE "x-directory/normal"
+#define BLOCK_DEVICE_TYPE "x-special/device-block"
 #define EXTINF "#EXTINF:"
 #define DEBUG(x) { if (parser->priv->debug != FALSE) x; }
 
@@ -230,6 +231,16 @@ my_gnome_vfs_get_mime_type_with_data (const char *uri, gpointer *data, TotemPlPa
 
 	*data = NULL;
 
+	/* Stat for a block device, we're screwed as far as speed
+	 * is concerned now */
+	if (g_str_has_prefix (uri, "file://") != FALSE) {
+		struct stat buf;
+		if (stat (uri + strlen ("file://"), &buf) == 0) {
+			if (S_ISBLK (buf.st_mode))
+				return BLOCK_DEVICE_TYPE;
+		}
+	}
+
 	/* Open the file. */
 	result = gnome_vfs_open (&handle, uri, GNOME_VFS_OPEN_READ);
 	if (result != GNOME_VFS_OK) {
@@ -238,6 +249,7 @@ my_gnome_vfs_get_mime_type_with_data (const char *uri, gpointer *data, TotemPlPa
 		DEBUG(g_print ("URL '%s' couldn't be opened in _get_mime_type_with_data: '%s'\n", uri, gnome_vfs_result_to_string (result)));
 		return NULL;
 	}
+	DEBUG(g_print ("URL '%s' was opened successfully in _get_mime_type_with_data:\n", uri));
 
 	/* Read the whole thing, up to MIME_READ_CHUNK_SIZE */
 	buffer = NULL;
@@ -288,6 +300,20 @@ my_gnome_vfs_get_mime_type_with_data (const char *uri, gpointer *data, TotemPlPa
 	return mimetype;
 }
 
+static char *
+totem_pl_parser_read_entire_file (const char *url, int *size)
+{
+	char *contents;
+
+	if (gnome_vfs_read_entire_file (url, size, &contents) != GNOME_VFS_OK)
+		return NULL;
+
+	contents = g_realloc ((gpointer) contents, *size + 1);
+	contents[*size] = '\0';
+
+	return contents;
+}
+
 static char*
 totem_pl_parser_base_url (const char *url)
 {
@@ -933,7 +959,8 @@ totem_pl_parser_add_ram (TotemPlParser *parser, const char *url, gpointer data)
 	int size, i;
 	const char *split_char;
 
-	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	contents = totem_pl_parser_read_entire_file (url, &size);
+	if (contents == NULL)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	/* figure out whether we're a unix or dos RAM file */
@@ -992,7 +1019,8 @@ totem_pl_parser_add_asf_reference_parser (TotemPlParser *parser,
 	char *contents, **lines, *ref, *split_char;
 	int size;
 
-	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	contents = totem_pl_parser_read_entire_file (url, &size);
+	if (contents == NULL)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	if (strstr(contents,"\x0d") == NULL) {
@@ -1036,7 +1064,8 @@ totem_pl_parser_add_asf_parser (TotemPlParser *parser,
 		return totem_pl_parser_add_asf_reference_parser (parser, url, data);
 	}
 
-	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	contents = totem_pl_parser_read_entire_file (url, &size);
+	if (contents == NULL)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	if (size <= 4) {
@@ -1185,7 +1214,8 @@ totem_pl_parser_add_pls (TotemPlParser *parser, const char *url, gpointer data)
 	char *contents;
 	int size;
 
-	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	contents = totem_pl_parser_read_entire_file (url, &size);
+	if (contents == NULL)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	if (size == 0) {
@@ -1234,7 +1264,8 @@ totem_pl_parser_add_m3u (TotemPlParser *parser, const char *url, gpointer data)
 	const char *split_char;
 	gboolean extinfo;
 
-	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	contents = totem_pl_parser_read_entire_file (url, &size);
+	if (contents == NULL)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	/* .pls files with a .m3u extension, the nasties */
@@ -1423,7 +1454,8 @@ totem_pl_parser_add_asx (TotemPlParser *parser, const char *url, gpointer data)
 	int size;
 	gboolean retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 
-	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	contents = totem_pl_parser_read_entire_file (url, &size);
+	if (contents == NULL)
 		return FALSE;
 
 	doc = xmlParseMemory (contents, size);
@@ -1449,6 +1481,26 @@ totem_pl_parser_add_asx (TotemPlParser *parser, const char *url, gpointer data)
 	return retval;
 }
 
+static TotemPlParserResult
+totem_pl_parser_add_block (TotemPlParser *parser, const char *url, gpointer data)
+{
+	MediaType type;
+	char *media_url;
+	GError *err = NULL;
+
+	type = totem_cd_detect_type_with_url (url, &media_url, &err);
+	if (err != NULL)
+		DEBUG(g_print ("Couldn't get CD type for URL '%s': %s\n", url, err->message));
+	if (type == MEDIA_TYPE_DATA || media_url == NULL)
+		return TOTEM_PL_PARSER_RESULT_UNHANDLED;
+	else if (type == MEDIA_TYPE_ERROR)
+		return TOTEM_PL_PARSER_RESULT_ERROR;
+
+	totem_pl_parser_add_one_url (parser, media_url, NULL);
+	g_free (media_url);
+	return TOTEM_PL_PARSER_RESULT_SUCCESS;
+}
+
 static TotemPlParserResult
 totem_pl_parser_add_ra (TotemPlParser *parser, const char *url, gpointer data)
 {
@@ -1547,7 +1599,8 @@ totem_pl_parser_add_smil (TotemPlParser *parser, const char *url, gpointer data)
 	int size;
 	TotemPlParserResult retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 
-	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	contents = totem_pl_parser_read_entire_file (url, &size);
+	if (contents == NULL)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	doc = xmlParseMemory (contents, size);
@@ -1605,7 +1658,8 @@ totem_pl_parser_add_quicktime_metalink (TotemPlParser *parser, const char *url,
 	char *contents = NULL, *src;
 	int size;
 
-	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	contents = totem_pl_parser_read_entire_file (url, &size);
+	if (contents == NULL)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	doc = xmlParseMemory (contents, size);
@@ -1667,7 +1721,8 @@ totem_pl_parser_add_desktop (TotemPlParser *parser, const char *url, gpointer da
 	const char *path, *display_name, *type;
 	int size;
 
-	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	contents = totem_pl_parser_read_entire_file (url, &size);
+	if (contents == NULL)
 		return TOTEM_PL_PARSER_RESULT_ERROR;
 
 	lines = g_strsplit (contents, "\n", 0);
@@ -1842,7 +1897,8 @@ totem_pl_parser_add_xspf (TotemPlParser *parser, const char *url,
 	int size;
 	gboolean retval = TOTEM_PL_PARSER_RESULT_UNHANDLED;
 
-	if (gnome_vfs_read_entire_file (url, &size, &contents) != GNOME_VFS_OK)
+	contents = totem_pl_parser_read_entire_file (url, &size);
+	if (contents == NULL)
 		return FALSE;
 
 	doc = xmlParseMemory (contents, size);
@@ -1944,6 +2000,7 @@ static PlaylistTypes special_types[] = {
 	{ "application/x-cd-image", totem_pl_parser_add_iso },
 	{ "application/x-cue", totem_pl_parser_add_cue },
 	{ "application/xspf+xml", totem_pl_parser_add_xspf},
+	{ "x-special/device-block", totem_pl_parser_add_block },
 };
 
 static PlaylistTypes ignore_types[] = {
@@ -1960,6 +2017,7 @@ static PlaylistTypes dual_types[] = {
 	{ "audio/x-pn-realaudio", totem_pl_parser_add_ra },
 	{ "application/vnd.rn-realmedia", totem_pl_parser_add_ra },
 	{ "audio/x-pn-realaudio-plugin", totem_pl_parser_add_ra },
+	{ "audio/vnd.rn-realaudio", totem_pl_parser_add_ra },
 	{ "text/plain", totem_pl_parser_add_ra },
 	{ "video/x-ms-asf", totem_pl_parser_add_asf },
 	{ "video/x-ms-wmv", totem_pl_parser_add_asf },
