@@ -34,12 +34,19 @@ GladeXML	*totem_interface_load		(const char *name,
 						 const char *display_name,
 						 gboolean fatal,
 						 GtkWindow *parent);
+GladeXML	*totem_interface_load_with_root (const char *name,
+						 const char *root_widget,
+						 const char *display_name,
+						 gboolean fatal,
+						 GtkWindow *parent);
 void		 totem_interface_error		(const char *title,
 						 const char *reason,
 						 GtkWindow *parent);
 void		 totem_interface_error_blocking	(const char *title,
 						 const char *reason,
 						 GtkWindow *parent);
+void		 totem_interface_set_transient_for (GtkWindow *window,
+						    GtkWindow *parent);
 
 G_END_DECLS
 
